@@ -16,6 +16,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "gtd-provider.h"
 #include "gtd-task-row.h"
 #include "gtd-task.h"
 #include "gtd-task-list.h"
@@ -23,8 +24,11 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-typedef struct
+struct _GtdTaskRow
 {
+  GtkListBoxRow      parent;
+
+  /*<private>*/
   GtkRevealer               *revealer;
   GtkStack                  *stack;
 
@@ -41,22 +45,26 @@ typedef struct
   GtkSpinner                *task_loading_spinner;
   GtkLabel                  *title_label;
 
+  /* dnd widgets */
+  GtkWidget                 *dnd_box;
+  GtkWidget                 *dnd_event_box;
+  GtkWidget                 *dnd_icon;
+  gdouble                    clicked_x;
+  gdouble                    clicked_y;
+
+  gboolean                   handle_subtasks : 1;
+
   /* data */
   gboolean                   new_task_mode;
   GtdTask                   *task;
-} GtdTaskRowPrivate;
 
-struct _GtdTaskRow
-{
-  GtkListBoxRow      parent;
-
-  /*<private>*/
-  GtdTaskRowPrivate *priv;
+  gint                       destroy_row_timeout_id;
 };
 
 #define PRIORITY_ICON_SIZE         8
+#define DND_ICON_SCALE             0.85
 
-G_DEFINE_TYPE_WITH_PRIVATE (GtdTaskRow, gtd_task_row, GTK_TYPE_LIST_BOX_ROW)
+G_DEFINE_TYPE (GtdTaskRow, gtd_task_row, GTK_TYPE_LIST_BOX_ROW)
 
 enum {
   ENTER,
@@ -68,13 +76,139 @@ enum {
 
 enum {
   PROP_0,
+  PROP_HANDLE_SUBTASKS,
   PROP_NEW_TASK_MODE,
   PROP_TASK,
   LAST_PROP
 };
 
+typedef enum
+{
+  CURSOR_NONE,
+  CURSOR_GRAB,
+  CURSOR_GRABBING
+} CursorType;
+
 static guint signals[NUM_SIGNALS] = { 0, };
 
+static void
+set_dnd_cursor (GtkWidget  *widget,
+                CursorType  type)
+{
+  GdkDisplay *display;
+  GdkCursor *cursor;
+
+  if (!gtk_widget_get_realized (widget))
+    return;
+
+  display = gtk_widget_get_display (widget);
+
+  switch (type)
+    {
+    case CURSOR_NONE:
+      cursor = NULL;
+      break;
+
+    case CURSOR_GRAB:
+      cursor = gdk_cursor_new_from_name (display, "grab");
+      break;
+
+    case CURSOR_GRABBING:
+      cursor = gdk_cursor_new_from_name (display, "grabbing");
+      break;
+
+    default:
+      cursor = NULL;
+    }
+
+  gdk_window_set_cursor (gtk_widget_get_window (widget), cursor);
+  gdk_display_flush (display);
+
+  g_clear_object (&cursor);
+}
+
+static cairo_surface_t*
+get_dnd_icon (GtdTaskRow *self)
+{
+  cairo_surface_t *surface;
+  GtkWidget *widget;
+  cairo_t *cr;
+  gint real_x;
+
+  widget = GTK_WIDGET (self);
+  gtk_widget_translate_coordinates (self->dnd_event_box,
+                                    gtk_widget_get_parent (widget),
+                                    0,
+                                    0,
+                                    &real_x,
+                                    NULL);
+
+  /* Make it transparent */
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+                                        gtk_widget_get_allocated_width (widget) * DND_ICON_SCALE,
+                                        gtk_widget_get_allocated_height (widget) * DND_ICON_SCALE);
+
+  cr = cairo_create (surface);
+  cairo_scale (cr, DND_ICON_SCALE, DND_ICON_SCALE);
+
+  gtk_widget_draw (widget, cr);
+
+  cairo_surface_set_device_offset (surface,
+                                   -(self->clicked_x + real_x) * DND_ICON_SCALE,
+                                   -self->clicked_y * DND_ICON_SCALE);
+
+  return surface;
+}
+
+static gboolean
+mouse_out_event (GtkWidget  *widget,
+                 GdkEvent   *event,
+                 GtdTaskRow *self)
+{
+  set_dnd_cursor (widget, CURSOR_NONE);
+
+  return GDK_EVENT_STOP;
+}
+
+static gboolean
+mouse_over_event (GtkWidget  *widget,
+                  GdkEvent   *event,
+                  GtdTaskRow *self)
+{
+  set_dnd_cursor (widget, CURSOR_GRAB);
+
+  return GDK_EVENT_STOP;
+}
+
+static gboolean
+button_press_event (GtkWidget      *widget,
+                    GdkEventButton *event,
+                    GtdTaskRow     *self)
+{
+  self->clicked_x = event->x;
+  self->clicked_y = event->y;
+
+  return GDK_EVENT_PROPAGATE;
+}
+
+static void
+drag_begin_cb (GtkWidget      *widget,
+               GdkDragContext *context,
+               GtdTaskRow     *self)
+{
+  cairo_surface_t *surface;
+
+  surface = get_dnd_icon (self);
+
+  set_dnd_cursor (widget, CURSOR_GRABBING);
+
+  gtk_drag_set_icon_surface (context, surface);
+
+  gtk_widget_hide (GTK_WIDGET (self));
+
+  g_clear_pointer (&surface, cairo_surface_destroy);
+}
+
 static void
 gtd_task_row__priority_changed_cb (GtdTaskRow *row,
                                    GParamSpec *spec,
@@ -115,6 +249,30 @@ gtd_task_row__priority_changed_cb (GtdTaskRow *row,
   gtk_widget_queue_draw (GTK_WIDGET (row));
 }
 
+static void
+complete_changed_cb (GtdTaskRow *self,
+                     GParamSpec *pspec,
+                     GtdTask    *task)
+{
+  GtkStyleContext *context;
+
+  context = gtk_widget_get_style_context (GTK_WIDGET (self));
+
+  if (gtd_task_get_complete (task))
+    gtk_style_context_add_class (context, "complete");
+  else
+    gtk_style_context_remove_class (context, "complete");
+}
+
+static void
+depth_changed_cb (GtdTaskRow *self,
+                  GParamSpec *pspec,
+                  GtdTask    *task)
+{
+  gtk_widget_set_margin_start (self->dnd_box,
+                               self->handle_subtasks ? 32 * gtd_task_get_depth (task) : 0);
+}
+
 static gboolean
 gtd_task_row__date_changed_binding (GBinding     *binding,
                                     const GValue *from_value,
@@ -188,10 +346,12 @@ gtd_task_row__create_task_for_name (const gchar *name)
   return task;
 }
 
-static void
-gtd_task_row__destroy_cb (GtkWidget *widget)
+static gboolean
+gtd_task_row__destroy_cb (GtkWidget *row)
 {
-  GTK_WIDGET_CLASS (gtd_task_row_parent_class)->destroy (widget);
+  gtk_widget_destroy (row);
+
+  return G_SOURCE_REMOVE;
 }
 
 GtkWidget*
@@ -207,35 +367,39 @@ gtd_task_row__entry_focus_out (GtkWidget     *widget,
                                GdkEventFocus *event,
                                gpointer       user_data)
 {
-  GtdTaskRowPrivate *priv = GTD_TASK_ROW (user_data)->priv;
+  GtdTaskRow *self = GTD_TASK_ROW (user_data);
 
   g_return_val_if_fail (GTD_IS_TASK_ROW (user_data), FALSE);
 
-  if (priv->new_task_mode)
-    gtk_stack_set_visible_child_name (priv->new_task_stack, "label");
+  if (self->new_task_mode)
+    gtk_stack_set_visible_child_name (self->new_task_stack, "label");
   else
-    gtk_stack_set_visible_child_name (priv->task_stack, "label");
+    gtk_stack_set_visible_child_name (self->task_stack, "label");
 
   return FALSE;
 }
 
 static gboolean
-gtd_task_row__focus_in (GtkWidget *widget,
+gtd_task_row__focus_in (GtkWidget     *widget,
                         GdkEventFocus *event)
 {
-  GtdTaskRowPrivate *priv = GTD_TASK_ROW (widget)->priv;
+  GtdTaskRow *self = GTD_TASK_ROW (widget);
 
   g_return_val_if_fail (GTD_IS_TASK_ROW (widget), FALSE);
 
-  if (priv->new_task_mode)
+  if (self->new_task_mode)
     {
-      gtk_stack_set_visible_child_name (priv->new_task_stack, "entry");
-      gtk_widget_grab_focus (GTK_WIDGET (priv->new_task_entry));
+      gtk_stack_set_visible_child_name (self->new_task_stack, "entry");
+      gtk_widget_grab_focus (GTK_WIDGET (self->new_task_entry));
+
+      g_signal_emit (self, signals[EXIT], 0);
     }
   else
     {
-      gtk_stack_set_visible_child_name (priv->task_stack, "title");
-      gtk_widget_grab_focus (GTK_WIDGET (priv->title_entry));
+      gtk_stack_set_visible_child_name (self->task_stack, "title");
+      gtk_widget_grab_focus (GTK_WIDGET (self->title_entry));
+
+      g_signal_emit (self, signals[ENTER], 0);
     }
 
   return FALSE;
@@ -245,15 +409,15 @@ static gboolean
 gtd_task_row__key_press_event (GtkWidget   *row,
                                GdkEventKey *event)
 {
-  GtdTaskRowPrivate *priv = GTD_TASK_ROW (row)->priv;
+  GtdTaskRow *self = GTD_TASK_ROW (row);
 
   if (event->keyval == GDK_KEY_Escape && // Esc is pressed
       !(event->state & (GDK_SHIFT_MASK|GDK_CONTROL_MASK))) // No modifiers together
     {
-      if (priv->new_task_mode)
+      if (self->new_task_mode)
         {
-          gtk_stack_set_visible_child_name (priv->new_task_stack, "label");
-          gtk_entry_set_text (priv->new_task_entry, "");
+          gtk_stack_set_visible_child_name (self->new_task_stack, "label");
+          gtk_entry_set_text (self->new_task_entry, "");
           return TRUE;
         }
       else
@@ -269,33 +433,64 @@ static void
 gtd_task_row__entry_activated (GtkEntry *entry,
                                gpointer  user_data)
 {
-  GtdTaskRowPrivate *priv = GTD_TASK_ROW (user_data)->priv;
+  GtdTaskRow *self = GTD_TASK_ROW (user_data);
 
   g_return_if_fail (GTD_IS_TASK_ROW (user_data));
   g_return_if_fail (GTK_IS_ENTRY (entry));
 
-  if (entry == priv->new_task_entry)
+  if (entry == self->new_task_entry)
     {
       GtdTask *new_task;
 
       /* Cannot create empty tasks */
-      if (gtk_entry_get_text_length (priv->new_task_entry) == 0)
+      if (gtk_entry_get_text_length (self->new_task_entry) == 0)
         return;
 
-      new_task = gtd_task_row__create_task_for_name (gtk_entry_get_text (priv->new_task_entry));
+      new_task = gtd_task_row__create_task_for_name (gtk_entry_get_text (self->new_task_entry));
 
       g_signal_emit (user_data, signals[CREATE_TASK], 0, new_task);
 
-      gtk_entry_set_text (priv->new_task_entry, "");
+      gtk_entry_set_text (self->new_task_entry, "");
     }
 }
 
 static void
 gtd_task_row_finalize (GObject *object)
 {
+  GtdTaskRow *self = GTD_TASK_ROW (object);
+
+  g_clear_object (&self->task);
+
   G_OBJECT_CLASS (gtd_task_row_parent_class)->finalize (object);
 }
 
+static void
+gtd_task_row_dispose (GObject *object)
+{
+  GtdTaskRow *self;
+  GtdTask *task;
+
+  self = GTD_TASK_ROW (object);
+  task = self->task;
+
+  if (task)
+    {
+      g_signal_handlers_disconnect_by_func (task,
+                                        depth_changed_cb,
+                                        self);
+
+      g_signal_handlers_disconnect_by_func (task,
+                                            complete_changed_cb,
+                                            self);
+
+      g_signal_handlers_disconnect_by_func (task,
+                                            gtd_task_row__priority_changed_cb,
+                                            self);
+    }
+
+  G_OBJECT_CLASS (gtd_task_row_parent_class)->dispose (object);
+}
+
 static void
 gtd_task_row_get_property (GObject    *object,
                            guint       prop_id,
@@ -306,12 +501,16 @@ gtd_task_row_get_property (GObject    *object,
 
   switch (prop_id)
     {
+    case PROP_HANDLE_SUBTASKS:
+      g_value_set_boolean (value, self->handle_subtasks);
+      break;
+
     case PROP_NEW_TASK_MODE:
-      g_value_set_boolean (value, self->priv->new_task_mode);
+      g_value_set_boolean (value, self->new_task_mode);
       break;
 
     case PROP_TASK:
-      g_value_set_object (value, self->priv->task);
+      g_value_set_object (value, self->task);
       break;
 
     default:
@@ -329,6 +528,10 @@ gtd_task_row_set_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_HANDLE_SUBTASKS:
+      gtd_task_row_set_handle_subtasks (self, g_value_get_boolean (value));
+      break;
+
     case PROP_NEW_TASK_MODE:
       gtd_task_row_set_new_task_mode (self, g_value_get_boolean (value));
       break;
@@ -350,26 +553,6 @@ gtd_task_row_activate (GtkListBoxRow *row)
   g_signal_emit (row, signals[ENTER], 0);
 }
 
-static void
-gtd_task_row_destroy (GtkWidget *widget)
-{
-  GtdTaskRow *row = GTD_TASK_ROW (widget);
-
-  if (!gtk_revealer_get_child_revealed (row->priv->revealer))
-    {
-      gtd_task_row__destroy_cb (GTK_WIDGET (row));
-    }
-  else
-    {
-      g_signal_connect_swapped (row->priv->revealer,
-                                "notify::child-revealed",
-                                G_CALLBACK (gtk_widget_destroy),
-                                row);
-
-      gtk_revealer_set_reveal_child (row->priv->revealer, FALSE);
-    }
-}
-
 static void
 gtd_task_row_class_init (GtdTaskRowClass *klass)
 {
@@ -377,16 +560,30 @@ gtd_task_row_class_init (GtdTaskRowClass *klass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
   GtkListBoxRowClass *row_class = GTK_LIST_BOX_ROW_CLASS (klass);
 
+  object_class->dispose = gtd_task_row_dispose;
   object_class->finalize = gtd_task_row_finalize;
   object_class->get_property = gtd_task_row_get_property;
   object_class->set_property = gtd_task_row_set_property;
 
-  widget_class->destroy = gtd_task_row_destroy;
   widget_class->focus_in_event = gtd_task_row__focus_in;
   widget_class->key_press_event = gtd_task_row__key_press_event;
 
   row_class->activate = gtd_task_row_activate;
 
+  /**
+   * GtdTaskRow::handle-subtasks:
+   *
+   * If the row consider the task's subtasks to adjust various UI properties.
+   */
+  g_object_class_install_property (
+          object_class,
+          PROP_HANDLE_SUBTASKS,
+          g_param_spec_boolean ("handle-subtasks",
+                                "If the row adapts to subtasks",
+                                "Whether the row adapts to the task's subtasks",
+                                TRUE,
+                                G_PARAM_READWRITE));
+
   /**
    * GtdTaskRow::new-task-mode:
    *
@@ -478,30 +675,44 @@ gtd_task_row_class_init (GtdTaskRowClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/todo/ui/task-row.ui");
 
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskRow, done_check);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskRow, stack);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskRow, new_task_entry);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskRow, new_task_stack);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskRow, revealer);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskRow, task_date_label);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskRow, task_list_label);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskRow, task_stack);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskRow, task_loading_spinner);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskRow, title_entry);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskRow, title_label);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, dnd_box);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, dnd_event_box);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, dnd_icon);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, done_check);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, stack);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, new_task_entry);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, new_task_stack);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, revealer);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, task_date_label);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, task_list_label);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, task_stack);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, task_loading_spinner);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, title_entry);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, title_label);
 
+  gtk_widget_class_bind_template_callback (widget_class, button_press_event);
+  gtk_widget_class_bind_template_callback (widget_class, drag_begin_cb);
+  gtk_widget_class_bind_template_callback (widget_class, mouse_out_event);
+  gtk_widget_class_bind_template_callback (widget_class, mouse_over_event);
   gtk_widget_class_bind_template_callback (widget_class, gtd_task_row__entry_activated);
   gtk_widget_class_bind_template_callback (widget_class, gtd_task_row__entry_focus_out);
 
-  gtk_widget_class_set_css_name (widget_class, "task-row");
+  gtk_widget_class_set_css_name (widget_class, "taskrow");
 }
 
 static void
 gtd_task_row_init (GtdTaskRow *self)
 {
-  self->priv = gtd_task_row_get_instance_private (self);
+  self->handle_subtasks = TRUE;
 
   gtk_widget_init_template (GTK_WIDGET (self));
+
+  /* The source of DnD is the drag icon */
+  gtk_drag_source_set (self->dnd_event_box,
+                       GDK_BUTTON1_MASK,
+                       NULL,
+                       0,
+                       GDK_ACTION_COPY);
 }
 
 /**
@@ -517,7 +728,7 @@ gtd_task_row_get_new_task_mode (GtdTaskRow *row)
 {
   g_return_val_if_fail (GTD_IS_TASK_ROW (row), FALSE);
 
-  return row->priv->new_task_mode;
+  return row->new_task_mode;
 }
 
 /**
@@ -527,8 +738,6 @@ gtd_task_row_get_new_task_mode (GtdTaskRow *row)
  *
  * Sets @row new task mode to @new_task_mode. It is up to the caller
  * to set GtdTaskRow::task to %NULL.
- *
- * Returns:
  */
 void
 gtd_task_row_set_new_task_mode (GtdTaskRow *row,
@@ -536,18 +745,18 @@ gtd_task_row_set_new_task_mode (GtdTaskRow *row,
 {
   g_return_if_fail (GTD_IS_TASK_ROW (row));
 
-  if (row->priv->new_task_mode != new_task_mode)
+  if (row->new_task_mode != new_task_mode)
     {
-      row->priv->new_task_mode = new_task_mode;
+      row->new_task_mode = new_task_mode;
 
       if (new_task_mode)
         {
-          gtk_stack_set_visible_child_name (GTK_STACK (row->priv->stack), "new");
+          gtk_stack_set_visible_child_name (GTK_STACK (row->stack), "new");
           gtd_task_row_reveal (row);
         }
       else
         {
-          gtk_stack_set_visible_child_name (GTK_STACK (row->priv->stack), "task");
+          gtk_stack_set_visible_child_name (GTK_STACK (row->stack), "task");
         }
 
       g_object_notify (G_OBJECT (row), "new-task-mode");
@@ -568,7 +777,7 @@ gtd_task_row_get_task (GtdTaskRow *row)
 {
   g_return_val_if_fail (GTD_IS_TASK_ROW (row), NULL);
 
-  return row->priv->task;
+  return row->task;
 }
 
 /**
@@ -578,54 +787,46 @@ gtd_task_row_get_task (GtdTaskRow *row)
  *
  * Sets the internal #GtdTask of @row. The task must be set to %NULL
  * before setting GtdObject::new-task-mode to %TRUE.
- *
- * Returns:
  */
 void
 gtd_task_row_set_task (GtdTaskRow *row,
                        GtdTask    *task)
 {
-  GtdTaskRowPrivate *priv;
-
   g_return_if_fail (GTD_IS_TASK_ROW (row));
 
-  priv = row->priv;
-
-  if (row->priv->task != task)
+  if (g_set_object (&row->task, task))
     {
-      row->priv->task = task;
-
       if (task)
         {
-          gtk_label_set_label (row->priv->task_list_label, gtd_task_list_get_name (gtd_task_get_list (task)));
+          gtk_label_set_label (row->task_list_label, gtd_task_list_get_name (gtd_task_get_list (task)));
 
           g_object_bind_property (task,
                                   "title",
-                                  priv->title_entry,
+                                  row->title_entry,
                                   "text",
                                   G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
           g_object_bind_property (task,
                                   "title",
-                                  priv->title_label,
+                                  row->title_label,
                                   "label",
                                   G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
 
           g_object_bind_property (task,
                                   "complete",
-                                  row->priv->done_check,
+                                  row->done_check,
                                   "active",
                                   G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
           g_object_bind_property (task,
                                   "ready",
-                                  priv->task_loading_spinner,
+                                  row->task_loading_spinner,
                                   "visible",
                                   G_BINDING_INVERT_BOOLEAN | G_BINDING_SYNC_CREATE);
 
           g_object_bind_property_full (task,
                                        "due-date",
-                                       priv->task_date_label,
+                                       row->task_date_label,
                                        "label",
                                        G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE,
                                        gtd_task_row__date_changed_binding,
@@ -642,6 +843,18 @@ gtd_task_row_set_task (GtdTaskRow *row,
                                     "notify::priority",
                                     G_CALLBACK (gtd_task_row__priority_changed_cb),
                                     row);
+
+          complete_changed_cb (row, NULL, task);
+          g_signal_connect_swapped (task,
+                                    "notify::complete",
+                                    G_CALLBACK (complete_changed_cb),
+                                    row);
+
+          depth_changed_cb (row, NULL, task);
+          g_signal_connect_swapped (task,
+                                    "notify::depth",
+                                    G_CALLBACK (depth_changed_cb),
+                                    row);
         }
 
       g_object_notify (G_OBJECT (row), "task");
@@ -654,8 +867,6 @@ gtd_task_row_set_task (GtdTaskRow *row,
  * @show_list_name: %TRUE to show the list name, %FALSE to hide it
  *
  * Sets @row's list name label visibility to @show_list_name.
- *
- * Returns:
  */
 void
 gtd_task_row_set_list_name_visible (GtdTaskRow *row,
@@ -663,7 +874,7 @@ gtd_task_row_set_list_name_visible (GtdTaskRow *row,
 {
   g_return_if_fail (GTD_IS_TASK_ROW (row));
 
-  gtk_widget_set_visible (GTK_WIDGET (row->priv->task_list_label), show_list_name);
+  gtk_widget_set_visible (GTK_WIDGET (row->task_list_label), show_list_name);
 }
 
 /**
@@ -671,13 +882,64 @@ gtd_task_row_set_list_name_visible (GtdTaskRow *row,
  * @row: a #GtdTaskRow
  *
  * Runs a nifty animation to reveal @row.
- *
- * Returns:
  */
 void
 gtd_task_row_reveal (GtdTaskRow *row)
 {
   g_return_if_fail (GTD_IS_TASK_ROW (row));
 
-  gtk_revealer_set_reveal_child (row->priv->revealer, TRUE);
+  gtk_revealer_set_reveal_child (row->revealer, TRUE);
+}
+
+/**
+ * gtd_task_row_destroy:
+ * @self: a #GtdTaskRow
+ *
+ * Destroy @self after hiding it.
+ */
+void
+gtd_task_row_destroy (GtdTaskRow *self)
+{
+  g_return_if_fail (GTD_IS_TASK_ROW (self));
+
+  if (!gtk_revealer_get_child_revealed (self->revealer))
+    {
+      gtk_widget_destroy (GTK_WIDGET (self));
+    }
+  else if (self->destroy_row_timeout_id == 0)
+    {
+      guint duration;
+
+      duration = gtk_revealer_get_transition_duration (self->revealer);
+
+      gtk_revealer_set_reveal_child (self->revealer, FALSE);
+      self->destroy_row_timeout_id = g_timeout_add (duration,
+                                                    (GSourceFunc) gtd_task_row__destroy_cb,
+                                                    self);
+    }
+}
+
+gboolean
+gtd_task_row_get_handle_subtasks (GtdTaskRow *self)
+{
+  g_return_val_if_fail (GTD_IS_TASK_ROW (self), FALSE);
+
+  return self->handle_subtasks;
+}
+
+void
+gtd_task_row_set_handle_subtasks (GtdTaskRow *self,
+                                  gboolean    handle_subtasks)
+{
+  g_return_if_fail (GTD_IS_TASK_ROW (self));
+
+  if (self->handle_subtasks == handle_subtasks)
+    return;
+
+  self->handle_subtasks = handle_subtasks;
+
+  gtk_widget_set_visible (self->dnd_box, handle_subtasks);
+  depth_changed_cb (self, NULL, self->task);
+
+  g_object_notify (G_OBJECT (self), "handle-subtasks");
 }
