@@ -24,11 +24,25 @@
 #include <libical/icaltime.h>
 #include <libical/icaltimezone.h>
 
+/**
+ * SECTION:gtd-task
+ * @short_description: a task
+ * @title:GtdTask
+ * @stability:Unstable
+ * @see_also:#GtdTaskList
+ *
+ * A #GtdTask is an object that represents a task. All #GtdTasks
+ * must be inside a #GtdTaskList.
+ */
+
 typedef struct
 {
   gchar           *description;
   GtdTaskList     *list;
   ECalComponent   *component;
+  GtdTask         *parent;
+  GList           *subtasks;
+  gint             depth;
 } GtdTaskPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtdTask, gtd_task, GTD_TYPE_OBJECT)
@@ -38,15 +52,133 @@ enum
   PROP_0,
   PROP_COMPLETE,
   PROP_COMPONENT,
+  PROP_DEPTH,
   PROP_DESCRIPTION,
   PROP_CREATION_DATE,
   PROP_DUE_DATE,
   PROP_LIST,
+  PROP_PARENT,
   PROP_PRIORITY,
   PROP_TITLE,
   LAST_PROP
 };
 
+enum
+{
+  SUBTASK_ADDED,
+  SUBTASK_REMOVED,
+  N_SIGNALS
+};
+
+static guint signals[N_SIGNALS] = { 0, };
+
+static gboolean
+share_same_ancestor (GtdTask *t1,
+                     GtdTask *t2)
+{
+  GtdTask *p1, *p2;
+  gboolean has_ancestor;
+
+  has_ancestor = FALSE;
+  p1 = t1;
+
+  for (p1 = t1; p1 != NULL; p1 = gtd_task_get_parent (p1))
+    {
+      for (p2 = t2; p2 != NULL; p2 = gtd_task_get_parent (p2))
+        {
+          if (p1 == p2)
+            {
+              has_ancestor = TRUE;
+              break;
+            }
+        }
+
+      if (has_ancestor)
+        break;
+    }
+
+  return has_ancestor;
+}
+
+static GtdTask*
+get_root_task (GtdTask *self)
+{
+  GtdTaskPrivate *priv = gtd_task_get_instance_private (self);
+  GtdTask *aux = self;
+
+  while (priv->parent)
+    {
+      aux = priv->parent;
+      priv = gtd_task_get_instance_private (aux);
+    }
+
+  return aux;
+}
+
+
+static gint
+compare_by_subtasks (GtdTask **t1,
+                     GtdTask **t2)
+{
+  GtdTask *task1, *task2;
+
+  if (!t1 && !t2)
+    return  0;
+  if (!t1)
+    return  1;
+  if (!t2)
+    return -1;
+
+  task1 = *t1;
+  task2 = *t2;
+
+  if (share_same_ancestor (task1, task2))
+    {
+      gint depth_difference;
+
+      depth_difference = ABS (gtd_task_get_depth (task1) - gtd_task_get_depth (task2));
+
+      if (depth_difference == 0)
+        return 0;
+
+      if (gtd_task_is_subtask (task1, task2))
+        {
+          return -1;
+        }
+      else if (gtd_task_is_subtask (task2, task1))
+        {
+          return 1;
+        }
+      else
+        {
+          gint i;
+
+          for (i = depth_difference; i > 0; i--)
+            {
+              if (gtd_task_get_depth (task1) > gtd_task_get_depth (task2))
+                task1 = gtd_task_get_parent (task1);
+              else
+                task2 = gtd_task_get_parent (task2);
+            }
+
+          *t1 = task1;
+          *t2 = task2;
+        }
+    }
+  else
+    {
+      /*
+       * If either t1 or t2 have a parent, we have to compare them
+       * solely based on the root task of the subtask tree.
+       */
+      *t1 = get_root_task (task1);
+      *t2 = get_root_task (task2);
+    }
+
+  return 0;
+}
+
+
 static GDateTime*
 gtd_task__convert_icaltime (const icaltimetype *date)
 {
@@ -65,6 +197,99 @@ gtd_task__convert_icaltime (const icaltimetype *date)
   return dt;
 }
 
+static void
+set_depth (GtdTask *self,
+           gint     depth)
+{
+  GtdTaskPrivate *priv = gtd_task_get_instance_private (self);
+  GList *l;
+
+  priv->depth = depth;
+  g_object_notify (G_OBJECT (self), "depth");
+
+  for (l = priv->subtasks; l != NULL; l = l->next)
+    set_depth (l->data, depth + 1);
+}
+
+static void
+real_add_subtask (GtdTask *self,
+                  GtdTask *subtask)
+{
+  GtdTaskPrivate *priv, *subtask_priv;
+  ECalComponentId *id;
+  ECalComponent *comp;
+  icalcomponent *ical_comp;
+  icalproperty *property;
+
+  priv = gtd_task_get_instance_private (self);
+
+  if (g_list_find (priv->subtasks, subtask))
+    return;
+
+  id = e_cal_component_get_id (priv->component);
+  subtask_priv = gtd_task_get_instance_private (subtask);
+  comp = subtask_priv->component;
+
+  /* First, remove the subtask from it's parent's subtasks list */
+  if (subtask_priv->parent)
+    gtd_task_remove_subtask (subtask_priv->parent, subtask);
+
+  ical_comp = e_cal_component_get_icalcomponent (comp);
+  property = icalcomponent_get_first_property (ical_comp, ICAL_RELATEDTO_PROPERTY);
+
+  if (property)
+    icalproperty_set_relatedto (property, id->uid);
+  else
+    icalcomponent_add_property (ical_comp, icalproperty_new_relatedto (id->uid));
+
+  /* Add to this task's list of subtasks */
+  priv->subtasks = g_list_prepend (priv->subtasks, subtask);
+
+  /* Update the subtask's parent property */
+  subtask_priv->parent = self;
+  g_object_notify (G_OBJECT (subtask), "parent");
+
+  /* And also the task's depth */
+  set_depth (subtask, priv->depth + 1);
+
+  e_cal_component_free_id (id);
+}
+
+static void
+real_remove_subtask (GtdTask *self,
+                     GtdTask *subtask)
+{
+  GtdTaskPrivate *priv, *subtask_priv;
+  icalcomponent *ical_comp;
+  icalproperty *property;
+
+  priv = gtd_task_get_instance_private (self);
+
+  if (!g_list_find (priv->subtasks, subtask))
+    return;
+
+  subtask_priv = gtd_task_get_instance_private (subtask);
+
+  /* Remove the parent link from the subtask's component */
+  ical_comp = e_cal_component_get_icalcomponent (subtask_priv->component);
+  property = icalcomponent_get_first_property (ical_comp, ICAL_RELATEDTO_PROPERTY);
+
+  if (!property)
+    return;
+
+  icalcomponent_remove_property (ical_comp, property);
+
+  /* Add to this task's list of subtasks */
+  priv->subtasks = g_list_remove (priv->subtasks, subtask);
+
+  /* Update the subtask's parent property */
+  subtask_priv->parent = NULL;
+  g_object_notify (G_OBJECT (subtask), "parent");
+
+  /* And also the task's depth */
+  set_depth (subtask, 0);
+}
+
 static void
 gtd_task_finalize (GObject *object)
 {
@@ -123,6 +348,7 @@ gtd_task_get_property (GObject    *object,
 {
   GtdTask *self = GTD_TASK (object);
   GtdTaskPrivate *priv = gtd_task_get_instance_private (self);
+  GDateTime *date;
 
   switch (prop_id)
     {
@@ -138,18 +364,28 @@ gtd_task_get_property (GObject    *object,
       g_value_set_boxed (value, gtd_task_get_creation_date (self));
       break;
 
+    case PROP_DEPTH:
+      g_value_set_uint (value, priv->depth);
+      break;
+
     case PROP_DESCRIPTION:
       g_value_set_string (value, gtd_task_get_description (self));
       break;
 
     case PROP_DUE_DATE:
-      g_value_set_boxed (value, gtd_task_get_due_date (self));
+      date = gtd_task_get_due_date (self);
+      g_value_set_boxed (value, date);
+      g_clear_pointer (&date, g_date_time_unref);
       break;
 
     case PROP_LIST:
       g_value_set_object (value, priv->list);
       break;
 
+    case PROP_PARENT:
+      g_value_set_object (value, priv->parent);
+      break;
+
     case PROP_PRIORITY:
       g_value_set_int (value, gtd_task_get_priority (self));
       break;
@@ -224,6 +460,9 @@ gtd_task_class_init (GtdTaskClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtdObjectClass *obj_class = GTD_OBJECT_CLASS (klass);
 
+  klass->subtask_added = real_add_subtask;
+  klass->subtask_removed = real_remove_subtask;
+
   object_class->finalize = gtd_task_finalize;
   object_class->get_property = gtd_task_get_property;
   object_class->set_property = gtd_task_set_property;
@@ -274,6 +513,22 @@ gtd_task_class_init (GtdTaskClass *klass)
                             G_TYPE_DATE_TIME,
                             G_PARAM_READABLE));
 
+  /**
+   * GtdTask::depth:
+   *
+   * The depth of the task inside the subtask tree.
+   */
+  g_object_class_install_property (
+        object_class,
+        PROP_DEPTH,
+        g_param_spec_uint ("depth",
+                           "Depth of the task",
+                           "The depth of the task inside the subtasks tree.",
+                           0,
+                           G_MAXUINT,
+                           0,
+                           G_PARAM_READABLE));
+
   /**
    * GtdTask::description:
    *
@@ -317,6 +572,20 @@ gtd_task_class_init (GtdTaskClass *klass)
                              GTD_TYPE_TASK_LIST,
                              G_PARAM_READWRITE));
 
+  /**
+   * GtdTask::parent:
+   *
+   * The parent of the task.
+   */
+  g_object_class_install_property (
+        object_class,
+        PROP_PARENT,
+        g_param_spec_object ("parent",
+                              "Parent of the task",
+                              "The GtdTask that is parent of this task.",
+                              GTD_TYPE_TASK,
+                              G_PARAM_READABLE));
+
   /**
    * GtdTask::priority:
    *
@@ -346,6 +615,39 @@ gtd_task_class_init (GtdTaskClass *klass)
                              "The title of the task",
                              NULL,
                              G_PARAM_READWRITE));
+
+  /* Signals */
+  /**
+   * GtdTask:subtask-added:
+   *
+   * Emited when a subtask is added to @self.
+   */
+  signals[SUBTASK_ADDED] = g_signal_new ("subtask-added",
+                                         GTD_TYPE_TASK,
+                                         G_SIGNAL_RUN_LAST,
+                                         G_STRUCT_OFFSET (GtdTaskClass, subtask_added),
+                                         NULL,
+                                         NULL,
+                                         NULL,
+                                         G_TYPE_NONE,
+                                         1,
+                                         GTD_TYPE_TASK);
+
+  /**
+   * GtdTask:subtask-removed:
+   *
+   * Emited when a subtask is removed from @self.
+   */
+  signals[SUBTASK_REMOVED] = g_signal_new ("subtask-removed",
+                                           GTD_TYPE_TASK,
+                                           G_SIGNAL_RUN_LAST,
+                                           G_STRUCT_OFFSET (GtdTaskClass, subtask_removed),
+                                           NULL,
+                                           NULL,
+                                           NULL,
+                                           G_TYPE_NONE,
+                                           1,
+                                           GTD_TYPE_TASK);
 }
 
 static void
@@ -642,6 +944,7 @@ gtd_task_get_due_date (GtdTask *task)
 {
   ECalComponentDateTime comp_dt;
   GtdTaskPrivate *priv;
+  GDateTime *date;
 
   g_return_val_if_fail (GTD_IS_TASK (task), NULL);
 
@@ -649,7 +952,9 @@ gtd_task_get_due_date (GtdTask *task)
 
   e_cal_component_get_due (priv->component, &comp_dt);
 
-  return gtd_task__convert_icaltime (comp_dt.value);
+  date = gtd_task__convert_icaltime (comp_dt.value);
+  e_cal_component_free_datetime (&comp_dt);
+  return date;
 }
 
 /**
@@ -728,8 +1033,7 @@ gtd_task_set_due_date (GtdTask   *task,
         g_object_notify (G_OBJECT (task), "due-date");
     }
 
-  if (current_dt)
-    g_date_time_unref (current_dt);
+  g_clear_pointer (&current_dt, g_date_time_unref);
 }
 
 /**
@@ -965,6 +1269,14 @@ gtd_task_compare (GtdTask *t1,
   if (!t2)
     return -1;
 
+  /*
+   * Zero, compare by subtask hierarchy
+   */
+  retval = compare_by_subtasks (&t1, &t2);
+
+  if (retval != 0)
+    return retval;
+
   /*
    * First, compare by ::complete.
    */
@@ -1045,3 +1357,166 @@ gtd_task_compare (GtdTask *t1,
 
   return retval;
 }
+
+/**
+ * gtd_task_get_parent:
+ * @self: a #GtdTask
+ *
+ * Retrieves the parent task of @self, or %NULL if none is set.
+ *
+ * Returns: (transfer none)(nullable): the #GtdTask that is parent of @self
+ */
+GtdTask*
+gtd_task_get_parent (GtdTask *self)
+{
+  GtdTaskPrivate *priv;
+
+  g_return_val_if_fail (GTD_IS_TASK (self), NULL);
+
+  priv = gtd_task_get_instance_private (self);
+
+  return priv->parent;
+}
+
+/**
+ * gtd_task_get_subtasks:
+ * @self: a #GtdTask
+ *
+ * Retrieves the subtasks of @self, or %NULL if it has no subtasks.
+ *
+ * Returns: (transfer container)(nullable)(element-type Gtd.Task): the subtasks of @self
+ */
+GList*
+gtd_task_get_subtasks (GtdTask *self)
+{
+  GtdTaskPrivate *priv;
+
+  g_return_val_if_fail (GTD_IS_TASK (self), NULL);
+
+  priv = gtd_task_get_instance_private (self);
+
+  return g_list_copy (priv->subtasks);
+}
+
+/**
+ * gtd_task_add_subtask:
+ * @self: a #GtdTask
+ * @subtask: the subtask to be added to @self
+ *
+ * Adds @subtask as a subtask of @self.
+ */
+void
+gtd_task_add_subtask (GtdTask *self,
+                      GtdTask *subtask)
+{
+  GtdTaskPrivate *priv;
+
+  g_return_if_fail (GTD_IS_TASK (self));
+  g_return_if_fail (GTD_IS_TASK (subtask));
+
+  priv = gtd_task_get_instance_private (self);
+
+  if (!g_list_find (priv->subtasks, subtask) && !gtd_task_is_subtask (subtask, self))
+    {
+      g_signal_emit (self, signals[SUBTASK_ADDED], 0, subtask);
+    }
+}
+
+/**
+ * gtd_task_remove_subtask:
+ * @self: a #GtdTask
+ * @subtask: the subtask to be removed to @self
+ *
+ * Removes @subtask from @self.
+ */
+void
+gtd_task_remove_subtask (GtdTask *self,
+                         GtdTask *subtask)
+{
+  GtdTaskPrivate *priv;
+
+  g_return_if_fail (GTD_IS_TASK (self));
+  g_return_if_fail (GTD_IS_TASK (subtask));
+
+  priv = gtd_task_get_instance_private (self);
+
+  if (g_list_find (priv->subtasks, subtask))
+    {
+      g_signal_emit (self, signals[SUBTASK_REMOVED], 0, subtask);
+    }
+}
+
+/**
+ * gtd_task_is_subtask:
+ * @self: a #GtdTask
+ * @subtask: a #GtdTask
+ *
+ * Checks if @subtask is a subtask of @self, directly or indirectly.
+ *
+ * Returns: %TRUE is @subtask is a subtask of @self, %FALSE otherwise
+ */
+gboolean
+gtd_task_is_subtask (GtdTask *self,
+                     GtdTask *subtask)
+{
+  GtdTask *aux;
+  GQueue *queue;
+  gboolean is_subtask;
+
+  g_return_val_if_fail (GTD_IS_TASK (self), FALSE);
+  g_return_val_if_fail (GTD_IS_TASK (subtask), FALSE);
+
+  aux = self;
+  queue = g_queue_new ();
+  is_subtask = FALSE;
+
+  do
+    {
+      GtdTaskPrivate *priv;
+      GList *l;
+
+      priv = gtd_task_get_instance_private (aux);
+
+      for (l = priv->subtasks; l != NULL; l = l->next)
+        {
+          /* Found it, no need to continue looping */
+          if (l->data == subtask)
+            {
+              is_subtask = TRUE;
+              break;
+            }
+
+          g_queue_push_tail (queue, l->data);
+        }
+
+      if (is_subtask)
+        break;
+
+      aux = g_queue_pop_head (queue);
+    }
+  while (aux);
+
+  g_queue_free (queue);
+
+  return is_subtask;
+}
+
+/**
+ * gtd_task_get_depth:
+ * @self: a #GtdTask
+ *
+ * Retrieves the depth of @self in the subtasks tree.
+ *
+ * Returns: the depth of the task.
+ */
+gint
+gtd_task_get_depth (GtdTask *self)
+{
+  GtdTaskPrivate *priv;
+
+  g_return_val_if_fail (GTD_IS_TASK (self), 0);
+
+  priv = gtd_task_get_instance_private (self);
+
+  return priv->depth;
+}
