@@ -34,7 +34,14 @@ struct _GtdTaskClass
 {
   GtdObjectClass parent;
 
-  gpointer       padding[10];
+  /*< signals >*/
+  void          (*subtask_added)                      (GtdTask              *self,
+                                                       GtdTask              *subtask);
+
+  void          (*subtask_removed)                    (GtdTask              *self,
+                                                       GtdTask              *subtask);
+
+  gpointer       padding[8];
 };
 
 GtdTask*            gtd_task_new                      (ECalComponent        *component);
@@ -80,6 +87,21 @@ void                gtd_task_save                     (GtdTask              *tas
 gint                gtd_task_compare                  (GtdTask              *t1,
                                                        GtdTask              *t2);
 
+GtdTask*            gtd_task_get_parent               (GtdTask              *self);
+
+GList*              gtd_task_get_subtasks             (GtdTask              *self);
+
+void                gtd_task_add_subtask              (GtdTask              *self,
+                                                       GtdTask              *subtask);
+
+void                gtd_task_remove_subtask           (GtdTask              *self,
+                                                       GtdTask              *subtask);
+
+gboolean            gtd_task_is_subtask               (GtdTask              *self,
+                                                       GtdTask              *subtask);
+
+gint                gtd_task_get_depth                (GtdTask              *self);
+
 G_END_DECLS
 
 #endif /* GTD_TASK_H */
